#import "ReminderController.h"
#import "Reminder.h"

@implementation ReminderController

- (void)applicationDidFinishLaunching:(NSNotification *)notification
{
    SEL clickSelector, timerSelector;
    NSImage *miniwindowImage;
    clickSelector = @selector(updateControls);
    timerSelector = @selector(reloadData);

    miniwindowImage = [[NSImage alloc] initWithContentsOfFile:@"MiniTimer.tiff"];
    [mainWindow setMiniwindowImage:miniwindowImage];

    reminderArray = [[NSMutableArray alloc] init];
    [remindersTable setDataSource:self];
    [remindersTable sizeLastColumnToFit];
    [remindersTable setAction:clickSelector];

    [repeatsSwitchBox setRefusesFirstResponder:YES];
    [absoluteSwitchBox setRefusesFirstResponder:YES];
    [timeField setRefusesFirstResponder:YES];

    [hoursField setDelegate:self];
    [minutesField setDelegate:self];
    [secondsField setDelegate:self];
    
    [mainWindow makeKeyAndOrderFront:self];

    [NSTimer scheduledTimerWithTimeInterval:1 target:self selector:timerSelector userInfo:nil repeats:YES];
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)theApplication
{
    return YES;
}

- (BOOL)applicationShouldTerminate:(NSApplication *)sender
{
    if([reminderArray count])
      {
        NSBeep();
        switch (NSRunAlertPanel(@"EggTimer Warning", @"There are still reminders in the list.\nAre you sure you want to quit?", @"Yes", @"No", nil))
          {
            case 0:
                [self showWindow:self];
                return NO;
                break;
            case 1:
                return YES;
                break;
          }

      }
    return YES;
}

- (void)showWindow:(id)sender
{
    [mainWindow makeKeyAndOrderFront:self];
}

- (void)reloadData
{
        [timeField setStringValue:[[NSDate date]
            descriptionWithCalendarFormat:@"%x - %H:%M:%S" timeZone:nil
            locale:nil]];
    [remindersTable reloadData];
}
- (int)numberOfRowsInTableView:(NSTableView *)theTableView
{
    return [reminderArray count];
}

- (id)tableView:(NSTableView *)theTableView objectValueForTableColumn: (NSTableColumn *)theColumn row:(int)rowIndex;
{
    int hours, minutes, seconds;
    if ([[theColumn identifier] isEqualToString:@"Reminders"])
      {
        return [[reminderArray objectAtIndex:rowIndex] stringValue];
      }
    else if ([[theColumn identifier] isEqualToString:@"Time"])
      {
        [[reminderArray objectAtIndex:rowIndex] remainingTime:&hours minutes:&minutes seconds:&seconds];
        if([[reminderArray objectAtIndex:rowIndex] getAbsolute])
          {
            return [NSString stringWithFormat:@"A %02d:%02d:%02d",hours,minutes,seconds];
          }
        else
          {
            return [NSString stringWithFormat:@"R %02d:%02d:%02d",hours,minutes,seconds];
          }
      }
    else if ([[theColumn identifier] isEqualToString:@"Repeats"])
      {
        if([[reminderArray objectAtIndex:rowIndex] getRepeats])
          {
            return @"Repeats";
          }
        else
          {
            return @"Once";
          }
      }
    else
      {
        return nil;
      }
}

- (void)updateControls
{
    int rowIndex;
    int hours, minutes, seconds;
    rowIndex = [remindersTable selectedRow];
    if([reminderArray count] < 1)
      {
        [deleteButton setEnabled:NO];
        [replaceButton setEnabled:NO];
        [deleteMenuItem setEnabled:NO];
        [replaceMenuItem setEnabled:NO];

       
      }
    else if(rowIndex > -1 && rowIndex < [reminderArray count])
      {
        [deleteButton setEnabled:YES];
        [replaceButton setEnabled:YES];
        [deleteMenuItem setEnabled:YES];
        [replaceMenuItem setEnabled:YES];
        [reminderInField setStringValue:[[reminderArray objectAtIndex:rowIndex] stringValue]];
        [repeatsSwitchBox setState:[[reminderArray objectAtIndex:rowIndex] getRepeats]];
        [absoluteSwitchBox setState:[[reminderArray objectAtIndex:rowIndex] getAbsolute]];
        if([[reminderArray objectAtIndex:rowIndex] getValid])
          {
            [[reminderArray objectAtIndex:rowIndex] remainingTime:&hours minutes:&minutes seconds:&seconds];
            [hoursField setStringValue:[NSString stringWithFormat:@"%02d",hours]];
            [minutesField setStringValue:[NSString stringWithFormat:@"%02d",minutes]];
            [secondsField setStringValue:[NSString stringWithFormat:@"%02d",seconds]];
            [hoursSlider setIntValue:hours];
            [minutesSlider setIntValue:minutes];
            [secondsSlider setIntValue:seconds];
         }
        else
         {
            [hoursField setStringValue:@"00"];
            [minutesField setStringValue:@"00"];
            [secondsField setStringValue:@"00"];
            [hoursSlider setIntValue:0];
            [minutesSlider setIntValue:0];
            [secondsSlider setIntValue:0];
         }
      }
}

- (void)replaceReminder:(id)sender
{
    Reminder *aReminder;
    int selectedRow = [remindersTable selectedRow];
    if (selectedRow > -1 && selectedRow < [reminderArray count])
      {
        aReminder = [reminderArray objectAtIndex:selectedRow];
        if(aReminder)
          {
            if([self addReminder:self atIndex:selectedRow])
              {
                [self deleteReminderIdenticalTo:aReminder];
              }
          }
      }
    else
      {
        NSBeep();
        NSRunAlertPanel(@"EggTimer Warning",@"Nothing to change!", @"OK", nil, nil);
        [remindersTable reloadData];
      }
    [self updateControls];
}

- (void)addReminder:(id)sender
{
    [self addReminder:sender atIndex:-1];
}

- (BOOL)addReminder:(id)sender atIndex:(int)rowIndex
{
    Reminder *aReminder;
    SEL aSelector;
    int hr, min, sec;
    aSelector = @selector(showReminderPanel:);
    aReminder = [[Reminder alloc] init];
    repeats=[repeatsSwitchBox state];
    absolute=[absoluteSwitchBox state];
    hr = [hoursField intValue];
    min = [minutesField intValue];
    sec = [secondsField intValue];
   
    if(absolute || hr != 0 || min != 0 || sec != 0)
      {
        [aReminder setReminder:hr :min :sec :self :aSelector :(BOOL)repeats :(BOOL)absolute :[reminderInField stringValue]];
        if(rowIndex > -1)
          {
            [reminderArray insertObject:aReminder atIndex:rowIndex];
          }
        else
          {
           [reminderArray addObject:aReminder];
          }
        
        [remindersTable reloadData];
        [self updateControls];
        return YES;
      }
    else
      {
        NSBeep();
        NSRunAlertPanel(@"EggTimer Warning",@"Delay time must be greater than zero!", @"OK", nil, nil);
        [self updateControls];
        return NO;
      }
}

- (void)deleteReminderIdenticalTo:(id)aReminder
{
    [aReminder cancelReminder];
    [reminderArray removeObjectIdenticalTo:aReminder];
    [remindersTable reloadData];
    [self updateControls];
}

- (void)deleteSelectedReminder:(id)sender
{
    int rowIndex = [remindersTable selectedRow];      
    if(rowIndex > -1)
      {
        [[reminderArray objectAtIndex:rowIndex] cancelReminder];
        [reminderArray removeObjectAtIndex:rowIndex];
      }
    else
      {
        NSBeep();
        NSRunAlertPanel(@"EggTimer Warning",@"Nothing to delete!", @"OK", nil, nil);
      }

    if(rowIndex == 0)
      {
        rowIndex = 1;
      }
    [remindersTable selectRow:(rowIndex-1) byExtendingSelection:NO];
    [remindersTable reloadData];
    [self updateControls];
}

- (void)setAbsoluteOrRelative:(id)sender
{
    if([sender state])
      {
        absolute = YES;
        [repeatsSwitchBox setState:NO];
      }
    else
      {
        absolute = NO;
      }
}

- (void)setRepeats:(id)sender
{
    if([sender state])
      {
        if([absoluteSwitchBox state])
          {
            NSBeep();
            [sender setState:NO];
          }
        else
          {
            repeats = YES;
          }
      }
    else
      {
        repeats = NO;
      }
}

- (void)showReminderPanel:(NSTimer *)sender
{
    Reminder *aReminder;
    aReminder = [sender userInfo];
    NSBeep();
    switch (NSRunAlertPanel(@"EggTimer Reminder", [aReminder stringValue], @"OK", @"Delete", nil))
      {
        case 0:
            [self deleteReminderIdenticalTo:aReminder];
            break;
      }
    [self updateControls];
}

- (void)textDidEndEditing:(NSNotification *)aNotification
{
    [self takeIntValueFrom:[[aNotification userInfo] objectForKey:@"NSFieldEditor"]];
    NSBeep();
}

- (void)takeIntValueFrom:(id)sender
{
    int hr, min, sec;
    if([sender isEqual:hoursSlider])
      {
        [hoursField setStringValue:[NSString stringWithFormat:@"%02d",[hoursSlider intValue]]];
      }
    else if([sender isEqual:minutesSlider])
      {
        [minutesField setStringValue:[NSString stringWithFormat:@"%02d",[minutesSlider intValue]]];
      }
    else if([sender isEqual:secondsSlider])
      {
        [secondsField setStringValue:[NSString stringWithFormat:@"%02d",[secondsSlider intValue]]];
      }
    else if([sender isEqual:hoursField])
      {
        hr = [hoursField intValue];
        if(hr < 24 && hr >-1)
          {
            [hoursSlider setIntValue:hr];
            [hoursField setStringValue:[NSString stringWithFormat:@"%02d",hr]];
          }
        else
          {
            [hoursField setStringValue:[NSString stringWithFormat:@"%02d",[hoursSlider intValue]]];
          }
        [hoursField selectText:self];
      }
    else if([sender isEqual:minutesField])
      {
        min = [minutesField intValue];
        if(min < 60 && min >-1)
          {
            [minutesSlider setIntValue:min];
            [minutesField setStringValue:[NSString stringWithFormat:@"%02d",min]];
          }
        else
          {
            [minutesField setStringValue:[NSString stringWithFormat:@"%02d",[minutesSlider intValue]]];
          }
        [minutesField selectText:self];
      }
    else if([sender isEqual:secondsField])
      {
        sec = [secondsField intValue];
        if(sec < 61 && sec >-1)
          {
            [secondsSlider setIntValue:sec];
            [secondsField setStringValue:[NSString stringWithFormat:@"%02d",sec]];
          }
        else
          {
            [secondsField setStringValue:[NSString stringWithFormat:@"%02d",[secondsSlider intValue]]];
          }
        [secondsField selectText:self];
      }
}


@end
